/******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************
 *
 * Summary:
 *   A view layout subclass of ContentPane with layout as the following:
 * 
 *     +--------+------------------+---------------------------+--------------------+
 *     |  icon  | left header node |         right header node | right actions area |
 *     |  node  +------------------+---------------------------+--------------------+
 *     |        | type node / content selector                                      |
 *     +--------+--------+----------------------------------------------------------+
 *     | attributes node | status node with icon                                    |
 *     +-----------------+----------------------------------------------------------+
 *     | summary node                                                               |
 *     +----------------------------------------------------------------------------+
 *     | left actions area                                                          |
 *     +----------------------------------------------------------------------------+
 *     |                                                                            |
 *     | main / alt content container node                                          |
 *     |                                                                            |
 *     +----------------------------------------------------------------------------+
 *
 *   The left header are used for in-progress indicator and the title.  The title
 *   can be editable if the attribute "titleEditable" is set to true.  CS Help icon
 *   is located to the right of the title and only visible if the attribute
 *   "contextHelpId" is set.
 *   
 *   The icon take CSS class with a 48x48 pixels image.  It is only visible if the
 *   attribute "iconClass" is set.
 *   
 *   The type node can be used for content selector when dual contents is used. Set
 *   the attribute "useDualContents" to true to have a main or alternate content
 *   container.  Labels must be specified for the content selector for selected and
 *   deselected content container.
 *   
 *   Attributes node area is an opened DIV for consumer to populate.  There are CSS
 *   classes defined for comsumer to use to layout the nodes properly.  The classes
 *   are "attributes-entry", "attributes-entry-label", "attributes-entry-value" for
 *   text value, and "attributes-entry-icon" for icon value.
 *   
 *   Status node with icon is used for server call return status.  There are API's
 *   to set the status message and icon.  Only the status label of "Success...",
 *   "Error...", or "Info..." is shown and the detail message is displayed as a
 *   tooltip when hover over.  The node is hidden when the status is cleared.
 *   
 *   Summary node is another opened DIV for consumer to set the innerHTML for simple
 *   message or append children nodes.
 *   
 *   The left actions area uses action links with icons.  Use the API addLeftViewAction() to
 *   create the action in the area.  The container area is hidden until action(s) is added.
 *   
 *   The right actions area at the top right corner uses plain HTML button.  use the
 *   API addRightViewAction to create action in the area.
 *   
 */

dojo.provide("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("dijit._Templated");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Textarea");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.InlineEditor");

(function() {

dojo.declare("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout",
[dijit.layout.ContentPane, dijit._Templated],
{
    templateString: '<div id="${id}" style="display: inline" dojoAttachPoint="domNode" tabindex="-1" class="com-ibm-asq-common-web-ui-view-header">' +
                    '  <div dojoAttachPoint="viewHeaderBackNode" class="view-header-back" tabindex="-1">' +
                    '    <div class="view-header-container" tabindex="-1">' +
                    '      <div dojoAttachPoint="artifactContainer" class="artifact-container">' +
                    '        <div dojoAttachPoint="iconNode" class="${iconClass}" style="float:left"></div>' +
                    '        <div tabindex="-1" class="artifact-node">' +
                    '          <div>' +
                    '            <div dojoAttachPoint="actionsAreaRight" class="actions-area-right" style="float:right"></div>' +
                    '            <div dojoAttachPoint="actionsAreaLeft" class="actions-area-left" style="float:right"></div>' +
                    '            <div dojoAttachPoint="viewHeaderRight" waiRole="region" class="view-right-header" style="float:right"></div>' +
                    '            <div dojoAttachPoint="progressIndicator" class="progress-indicator" style="display: none">' +
                    '              <div class="view-progress-image">' +
                    '                <span dojoAttachPoint="progressLabel" class="view-progress-label"></span>' +
                    '              </div>' +
                    '            </div>' +
                    '            <div dojoAttachPoint="viewHeaderLeft" class="view-left-header">' +
                    '              <span dojoAttachPoint="viewTitle" class="view-title"></span>' +
                    '              <span dojoAttachPoint="viewSubTitle" class="view-title"></span>' +
                    '              <span dojoAttachPoint="viewEditableTitle" tabIndex="1" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditBox"' +
                    '                    value="${title}" class="view-title"></span>' +
                    '              <span dojoAttachPoint="viewHelpButtonNode" class="help-button-node"></span>' +
                    '            </div>' +
                    '          </div>' +
                    '          <div style="float:right;clear:right;width:auto;position:relative;" dojoAttachPoint="wholestatusContainer"> ' +
                    '            <span dojoAttachPoint="statusContainer">' +
                    '              <img dojoAttachPoint="statusSuccessNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusErrorNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusWarningNode" class="view-status-icon"></img>' +
                    '              <img dojoAttachPoint="statusInfoNode" class="view-status-icon"></img>' +
                    '              <span dojoAttachPoint="statusLineNode" class="view-status-value">&nbsp;</span>' +
                    '            </span>' +
                    '          </div> ' +
                    '          <div>' +
                    '            <span dojoAttachPoint="typeNode" class="view-artifact-type">artifact type</span>' +
                    '            <span dojoAttachPoint="contentSelectorNode" class="view-artifact-type">' +
                    '              <span dojoAttachPoint="mainContentSelector" dojoAttachEvent="onclick:_selectMainContent,onkeypress:_onKeySelectMainContent"></span>' +
                    '              <span class="content-selector-separator">&nbsp;|&nbsp;</span>' +
                    '              <span dojoAttachPoint="altContentSelector" dojoAttachEvent="onclick:_selectAltContent,onkeypress:_onKeySelectAltContent"></span>' +
                    '            </span>' +
                    '          </div>' +
                    '        </div>' +
                    '      </div>' +
                    '      <div class="attributes-node" dojoAttachPoint="attributesContainerNode">' +
                    '        <table>' +
                    '          <tr>' +
                    '            <td width="100%">' +
                    '              <div dojoAttachPoint="attributesNode"></div>' +
                    '            </td>' +
                    '            <td>' +
                    '              <div dojoAttachPoint="attributesNodeExt"></div>' +
                    '            </td>' +
                    '          </tr>' +
                    '        </table>' +
                    '      </div>' +
                    '      <div dojoAttachPoint="summaryNode" class="summary-node">' +
                    '        <span dojoAttachPoint="viewSummaryLabel" class="summary-label" style="float:left"></span>' +
                    '        <span class="attributes-entry-value"></span>' +
                    '        <span dojoAttachPoint="viewSummary" class="attributes-entry-value"></span>' +
                    '        <span dojoAttachPoint="viewEditableSummary" dojoType="com.ibm.asq.common.web.ui.internal.widgets.InlineEditArea"' +
                    '              class="summary-value">' +
                    '        </span>' +
                    '      </div>' +
                    '    </div>' +
                    '  </div>' +
					'  <div class="view-header-separator" dojoAttachPoint="viewHeaderSeparator">' +
					'    <div class="view-header-separator-l"></div>' +
					'    <div class="view-header-separator-r"></div>' +
					'  </div>' +
                    '  <div dojoAttachPoint="containerNode" waiRole="region" class="view-container"></div>' +
                    '  <div dojoAttachPoint="altContainerNode" waiRole="region" class="view-container" style="display:none"></div>' +
                    '</div>',

    titleEditable: false,
    contextHelpId: null,
    csHelpEngine: null,
	cshelp: null,
    csHelpProviderId: null,
    iconClass: null,
    
    noTitleLabel: "",  // prompt label for editable title when no value is specified
    noSummaryLabel: "",  // prompt label for editable summary when no value is specified
    summaryAsDesc: true, // The summary node is mostly used for description so the no value prompt
                         // string is default to say description.  Set this to false to have the
                         // default no value prompt string to say summary.
    title: "",
	maxTitleLength: 250,  // Maximum number of characters that can be entered in the editable title in line edit box
	maxSummaryLength: 0,  // Maximum number of characters that can be entered in the summary, 0 for no limit
	subTitle: "",
    typeLabel: "",
    summaryEditable: false,
    summaryLabel: "",
    summary: "",
    noline: false,
    
    useDualContents: false,
    defaultMainContent: true,
    // consumer should specify these labels
    //
    mainContentSelectedLabel: "",
    mainContentDeselectedLabel: "",
    altContentSelectedLabel: "",
    altContentDeselectedLabel: "",
    
    widgetsInTemplate: true,
    statusPopup: null,
    
    postCreate: function() {
        this.inherited('postCreate', arguments);

		//Set the max characters on the editable title
		this.viewEditableTitle.editorParams = { maxLength: this.maxTitleLength };
        //Set the max characters on the editable summary
        this.viewEditableSummary.editorParams = { maxTextAreaLength: this.maxSummaryLength };
        
        // server return status in hidden until a status is set
        //
        this.statusContainer.style.display = "none";
        
        this.statusSuccessNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/success_obj.gif");
        this.statusInfoNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/info_obj.gif");
        this.statusErrorNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
        this.statusWarningNode.src = com.ibm.asq.common.web.ui.internal.view.common.ImageCache.WARNING_ICON;
        this.statusPopup = new dijit.Tooltip({label: "", showDelay: 100, connectId:[this.statusLineNode]});
        
        this.setNoValueEditableTitleLabel(this.noTitleLabel);
        this.setNoValueEditableSummaryLabel(this.noSummaryLabel);
        
        this.setTitleEditable(this.titleEditable);
        this.setTitle(this.title);
        this.setSubTitle(this.subTitle);
        
        this.setTypeLabel(this.typeLabel);
        this.setIconClass(this.iconClass);
        this.setCSHelpId(this.contextHelpId, this.csHelpProviderId, this.csHelpEngine);

        this.setSummaryEditable(this.summaryEditable);
        this.setSummaryLabel(this.summaryLabel);
        this.setSummary(this.summary);

        // set up nodes for dual contents support
        this.typeNode.style.display = this.useDualContents ? "none" : "inline";
        this.contentSelectorNode.style.display = this.useDualContents ? "inline" : "none";

        if (this.useDualContents) {
            if (this.mainContentSelectedLabel == "")
                this.mainContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentSelected");
            if (this.mainContentDeselectedLabel == "")
                this.mainContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderMainContentDeselected");
            if (this.altContentSelectedLabel == "")
                this.altContentSelectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentSelected");
            if (this.altContentDeselectedLabel == "")
                this.altContentDeselectedLabel = MessageUtil.getMessage(MessageBundle.COMMON, "sampleLabel_viewHeaderAltContentDeselected");

            if (this.defaultMainContent) {
                this._selectMainContent();
            } else {
                this._selectAltContent();
            }
        }
        
        if( this.noline )
        {
        	this.viewHeaderBackNode.className = "view-header-back-no-line";
        }
		
		//set icon node to the right in RTL
		var lang = dojo.locale.split('-')[0];
		var RTL = {ar:true,fa:true,he:true,ur:true,yi:true,iw:true};
		if(RTL[lang]) {
			this.iconNode.style.cssFloat = "right";
			this.actionsAreaRight.style.cssFloat = "right";
			this.actionsAreaLeft.style.cssFloat = "right";
			this.viewHeaderRight.style.cssFloat = "right";
			this.wholestatusContainer.style.cssFloat = "left";
			this.statusContainer.style.cssFloat = "right";
			this.viewSummaryLabel.style.cssFloat = "right";
			
		}
    },

    // set the no value prompt label for editable title
    //
    setNoValueEditableTitleLabel: function(label) {
        this.noTitleLabel = label;
        if (label) {
            this.viewEditableTitle.setNoValuePrompt(null, null, label);
        }
    },
    
    // change handler for editable title
    //
    editableTitleChange: function(value) {
		var val = dojo.trim( value );
		this.setTitle(val);
//		this.onTitleEdited(val);
    },
    
    setTitleEditable: function(editable) {
        this.titleEditable = editable;
        this.viewEditableTitle.domNode.style.display = this.titleEditable ? "inline" : "none";
        this.viewTitle.style.display = this.titleEditable ? "none" : "inline";
        this.viewSubTitle.style.display = this.titleEditable ? "none" : "inline";

        if (this.titleEditable) {
            this.viewEditableTitle.onChange = dojo.hitch(this, "editableTitleChange");			
			dojo.connect(this.viewEditableTitle, "fireChangeEvent", this, "onTitleEdited");
            this.viewEditableTitle.onEditCallback = dojo.hitch(this, "hideCSHelp");
            this.viewEditableTitle.onEditCompleteCallback = dojo.hitch(this, "showCSHelp");
        }
    },

    // set the title to display in the header
    //
    setTitle: function(title) {
        this.title = title;
        if (this.viewTitle)
            this.viewTitle.innerHTML = this.title ? this.title : "";
        if (this.viewEditableTitle) {
			this.viewEditableTitle.value = (this.title ? this.title : "");
			this.viewEditableTitle._setDisplayValue(this.title ? this.title : "");
		}
    },
	
	setSubTitle: function(subTitle) {
        this.subTitle = subTitle;
        if (this.viewSubTitle)
            this.viewSubTitle.innerHTML = this.subTitle ? "&nbsp;" + this.subTitle : "";
	},
    
    getTitle: function() {
        return this.title;
    },
    
    setTypeLabel: function(type) {
        this.typeLabel = type;
        if (this.typeNode)
            this.typeNode.innerHTML = this.typeLabel ? this.typeLabel : "";
    },
    
    setIconClass: function(iconClass) {
        this.iconClass = iconClass;
        if (this.iconNode) {
            this.iconNode.className = this.iconClass ? this.iconClass : "";
            this.iconNode.style.paddingRight = "10px";
        }
    },
    
    setCSHelpId: function(helpId, csHelpProviderId, csHelpEngine){
        this.contextHelpId = helpId;
        this.csHelpProviderId = csHelpProviderId;
        this.csHelpEngine = csHelpEngine;
        if (this.viewHelpButtonNode) {
            dojox.data.dom.removeChildren(this.viewHelpButtonNode);
            if (this.contextHelpId) {
                this.viewHelpButtonNode.style.display = "inline";
                var args = {
                    'contextID': helpId,
                    'csHelpEngine': this.csHelpEngine,
                    'csHelpProviderId': this.csHelpProviderId
                };
                this.cshelp = new com.ibm.asq.common.web.ui.internal.util.CSHelp(args);
                this.viewHelpButtonNode.appendChild(this.cshelp.domNode);
            } else {
                this.viewHelpButtonNode.style.display = "none";
            }
        }
    },
    
    hideCSHelp: function() {
        this.viewHelpButtonNode.style.display = "none";
    },
    
    showCSHelp: function() {
        if (this.contextHelpId) {
            this.viewHelpButtonNode.style.display = "inline";
        }
    },

	setCSHelpButtonLabel: function(/* REQUIRED */ cshelpLabel) {
		if(this.cshelp)
			this.cshelp.setButtonLabel(cshelpLabel);
	},

    // set the no value prompt label for editable title
    //
    setNoValueEditableSummaryLabel: function(label) {
        this.noSummaryLabel = label;
        if (!label) {
            if (this.summaryAsDesc) 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterDesc");
            else 
                label = MessageUtil.getMessage(MessageBundle.COMMON, "promptLabel_clickEnterSummary");
        }
        this.viewEditableSummary.setNoValuePrompt(null, null, label);
    },
    
    setSummaryEditable: function(editable) {
        this.summaryEditable = editable;
        this.viewEditableSummary.domNode.style.display = this.summaryEditable ? "inline" : "none";
        this.viewSummary.style.display = this.summaryEditable ? "none" : "inline";
        if (this.summaryEditable) {
            this.viewEditableSummary.onChange = dojo.hitch(this, "editableSummaryChange");
        }
    },

    editableSummaryChange: function(value) {
        this.setSummary(value);
        this.onSummaryEdited(value);
    },
    
    setSummaryLabel: function(label) {
        this.summaryLabel = label;
        this.viewSummaryLabel.innerHTML = label ? label : "";
        this.viewSummaryLabel.style.display = label ? "inline" : "none";
    },
    
    setSummary: function(summary) {
        this.summary = summary;
        if (this.viewSummary) {
            this.viewSummary.innerHTML = this.summary ? this.summary : "";
        }
        if (this.viewEditableSummary) {
            if (this.viewEditableSummary.editing) {
                // cancel the editing when consumer explicitly set the summary value
                this.viewEditableSummary.cancel();
            }
            this.viewEditableSummary._setDisplayValue(this.summary ? this.summary : "");
            this.viewEditableSummary.value = this.summary ? this.summary : "";
        }
    },
    
    getSummary: function() {
        if (this.viewEditableSummary.editing) {
            // save the edited value implicitly when getting the summary value i.e. on consumer save
            this.viewEditableSummary.save();
        }
        return this.summary;
    },
    
    // Summary:
    //    set the status message in the header. Optionally, provide a type "Success", "Error",
    //    or "Info" parameter to qualify the status. 
    //
    setStatus: function(status, type, /*optional*/details) {
        this.statusContainer.style.display = "inline";
        
        var stText = status;
        var detailText = details;
        
        // stop the status line from extending too far off to the
        // left. Cap its length at 40. Show the full text in the tooltip.
        //
        if((status.length - 40) > 5) {
            stText = status.substr(0, 40) + "...";
            if(details) {
                detailText = status + "\n" + details;
            }
        }
        // detail message is in the tooltip popup
        this.statusPopup.label = (details) ? detailText : status;

        this.statusSuccessNode.style.display = (type == ViewHeaderLayout.SUCCESS) ? "inline" : "none";
        this.statusErrorNode.style.display = (type == ViewHeaderLayout.ERROR) ? "inline" : "none";
        this.statusInfoNode.style.display = "none";
        this.statusWarningNode.style.display = (type == ViewHeaderLayout.WARNING) ? "inline" : "none";
        
         this.statusLineNode.innerHTML = stText;
    },
   
    // set the status message in the header
    //
    clearStatus: function() {
        this.setStatus("");
        this.statusContainer.style.display = "none";
    },

    // Summary:
    //    show, or hide the content area DIV of the ViewHeaderLayout. 
    // 
    showContentArea: function(showContent /* true to show, false to hide */) {
        this.containerNode.style.display = (showContent) ? "inline" : "none";
    },
    
    // show the in-progress indicator and hide the title. Optionally, specify
    // whether the progress should hide the view's contents. If not specified, always shows
    // all contents. By hiding the view's content while the progress is showing,
    // this reduces the flickering while content is being loaded. When the progress is
    // ended all hidden content are shown.
    //
    setProgressStart: function(message, hideContent) {
        if (hideContent) {
            dojo.style(this.attributesContainerNode, "visibility", "hidden");
            dojo.style(this.typeNode, "visibility", "hidden");
            dojo.style(this.contentSelectorNode, "visibility", "hidden");
            dojo.style(this.summaryNode, "visibility", "hidden");
            dojo.style(this.actionsAreaRight, "visibility", "hidden");
            dojo.style(this.actionsAreaLeft, "visibility", "hidden");
            dojo.style(this.wholestatusContainer, "visibility", "hidden");
            dojo.style(this.viewHeaderSeparator, "visibility", "hidden");
            
            this.progressHideContent = true;
        }
        this.progressLabel.innerHTML = message;
        this.viewHeaderLeft.style.display = "none";
        this.progressIndicator.style.display = "inline";
    },
    
    // restore the title and hide the in-progress indicator
    //
    setProgressEnd: function() {
        if(this.progressHideContent) {
            dojo.style(this.attributesContainerNode, "visibility", "visible");
            dojo.style(this.typeNode, "visibility", "visible");
            dojo.style(this.contentSelectorNode, "visibility", "visible");
            dojo.style(this.summaryNode, "visibility", "visible");
            dojo.style(this.actionsAreaRight, "visibility", "visible");
            dojo.style(this.actionsAreaLeft, "visibility", "visible");
            dojo.style(this.wholestatusContainer, "visibility", "visible");
            dojo.style(this.viewHeaderSeparator, "visibility", "visible");

            this.progressHideContent = false;
        }
        this.progressLabel.innerHTML = "";
        this.viewHeaderLeft.style.display = "inline";
        this.progressIndicator.style.display = "none";
    },

    _addViewAction: function(label, handler, callbackData, iconClass, left, showActionLabel) {
        var arg = {
            label: label,
            showLabel: showActionLabel,
            'class': left ? "left-action" : "right-action",
            onClick: function( e ) {
                this.clientCallback(this.callbackData, e);
            }
        };
        
        if (iconClass) {
            arg["iconClass"] = iconClass;
        }
        var button = new dijit.form.Button(arg);
        
        // store the client callback and data into the button
        button.clientCallback = handler;
        button.callbackData = callbackData;
        
        if (left)
            this.getleftActionsContainer().appendChild(button.domNode);
        else
            this.getRightActionsContainer().appendChild(button.domNode);
            
        return button;
    },

    addLeftViewAction: function(label, handler, callbackData, iconClass) {
        this.actionsAreaLeft.style.display = "block";
        return this._addViewAction(label, handler, callbackData, iconClass, true, false);
    },
    
    addRightViewAction: function(label, handler, callbackData, iconClass) {
//        return this._addViewAction(label, handler, callbackData, iconClass, false, true);

//  use plain button at the moment
        var button = document.createElement('button');
        button.innerHTML = label;
        button.className = "right-action";
        button.onclick = function( e ) { 
			this.focus();
			this.clientCallback(this.callbackData, e); 
		};
		dojo.connect(button, "onmousedown", dojo.stopEvent);
        button.tabIndex = "2";
		
        // store the client callback and data into the button
        button.clientCallback = handler;
        button.callbackData = callbackData;
        this.getRightActionsContainer().appendChild(button);
        
        return button;
    },
    
    // return the right header DIV
    //
    getRightHeader: function() {
        return this.viewHeaderRight;
    },
    
    // return the main content area DIV
    //
    getContentNode: function() {
        return this.containerNode;
    },
    
    // return the alternate content area DIV
    //
    getAltContentNode: function() {
        return this.altContainerNode;
    },
    
    _selectMainContent: function() {
        if (this.useDualContents) {
            this.defaultMainContent = true;
            if (this.containerNode && this.altContainerNode) {
                this.mainContentSelector.innerHTML = this.mainContentSelectedLabel;
                this.mainContentSelector.className = "content-selector-selected";
                this.altContentSelector.innerHTML = this.altContentDeselectedLabel;
                this.altContentSelector.className = "content-selector-deselected";
                
                this.containerNode.style.display = "block";
                this.altContainerNode.style.display = "none";
				
				//set which content is selected
				this._mainContentSelected = true;
				this._altContentSelected = false;
            }
        }
    },
	
    isMainContentSelected: function() {
        return this._mainContentSelected;
    },	

    _selectAltContent: function() {
        if (this.useDualContents) {
            this.defaultMainContent = false;
            if (this.containerNode && this.altContainerNode) {
                this.mainContentSelector.innerHTML = this.mainContentDeselectedLabel;
                this.mainContentSelector.className = "content-selector-deselected";
                this.altContentSelector.innerHTML = this.altContentSelectedLabel;
                this.altContentSelector.className = "content-selector-selected";
                
                this.containerNode.style.display = "none";
                this.altContainerNode.style.display = "block";
				
				//set which content is selected
				this._altContentSelected = true;
                this._mainContentSelected = false;                			
            }
        }
    },

    isAltContentSelected: function() {
        return this._altContentSelected;
    },
	
    _onKeySelectMainContent: function(evt) {
        if(evt.keyCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE) {
            this._selectMainContent();
        }
    },
    
    _onKeySelectAltContent: function(evt) {
        if(evt.keyCode == dojo.keys.ENTER || evt.charCode == dojo.keys.SPACE) {
            this._selectAltContent();
        }
    },
    
    getInfoAttributesNode: function() {
        return this.attributesNode;
    },
    
    getExtInfoAttributesNode: function() {
        return this.attributesNodeExt;
    },
    
    addInfoLineBreak: function(/* optional */ position) {
        var br = document.createElement("span");
        br.innerHTML = "<br>";
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(br);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(br, childrenAttributes[position]);
        }
    },
    
    _createInfoAttributeNode: function(labelDomNode, valueDomNode) {
        var valueNode = valueDomNode;
        if( valueNode.nodeType == 3 )
        {
            //Cannot set the className on text nodes ( type 3 ) in IE
            valueNode = document.createElement( "span" );
            valueNode.appendChild( valueDomNode );
        }
        
        var attributeContainer = document.createElement("span");
        attributeContainer.className = "attributes-entry";
        dojo.addClass(labelDomNode, "attributes-entry-label");
        dojo.addClass(valueNode, "attributes-entry-value");
       
        attributeContainer.appendChild(labelDomNode);
        attributeContainer.appendChild(valueNode);
        return attributeContainer;
    },
    
    addInfoAttribute: function(labelDomNode, valueDomNode, /* optional */ position) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        var childrenAttributes = this.attributesNode.childNodes;
        if (!position || (childrenAttributes.length < 1) || (position >= childrenAttributes.length)) {
            // append to the end if position is not specified or no children attribute
            // or position is out of range
            this.attributesNode.appendChild(attributeContainer);
        } else {
            // insert before the child attribute of specified position
            this.attributesNode.insertBefore(attributeContainer, childrenAttributes[position]);
        }
		return attributeContainer;
    },
    
    setExtInfoAttribute: function(labelDomNode, valueDomNode) {
        var attributeContainer = this._createInfoAttributeNode(labelDomNode, valueDomNode);
        dojox.data.dom.removeChildren(this.attributesNodeExt);
        this.attributesNodeExt.appendChild(attributeContainer);
        return attributeContainer;
    },
    
    getSummaryNode: function() {
        return this.summaryNode;
    },

    getleftActionsContainer: function() {
        return this.actionsAreaLeft;
    },

    getRightActionsContainer: function() {
        return this.actionsAreaRight;
    },
    
    onTitleEdited: function(value) {
        // called when the title (editable) has been modified by the user.    
    },

    onSummaryEdited: function(value) {
        // called when the summary (editable) has been modified by the user.    
    }
});

//	Create some definitions that will be used in other files
//	Note that the ViewHeaderLayout variable is used locally to this file as a shortcut.
	var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
	ViewHeaderLayout.ERROR 		= 1;
	ViewHeaderLayout.SUCCESS 	= 2;
	ViewHeaderLayout.INFO 	= 3;
	ViewHeaderLayout.WARNING	= 4;
})();