/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//
//    The TemplatedAssetContentProvider lays out the asset's form based on a template.
//    A template is simply a structure that describes the sections and the fields to be shown
//    in each of the section. The structure of the template is as follows:
//  
// {
//  assetType: "Machine",
//  sections: [{
//        id: "Machine.details",
//        title: "Machine",
//        nColumns: 2,
//      actions: [], 
//        groups: [{
//            hidden: true,
//            root: "Machine",
//            attributes: ["ipAddress", "Manufacturer", "memory", "cpu", "owner"]
//        }]
//    }, {
//        id: "os.details",
//        title: "Operating System",
//        nColumns: 2,
//      actions: ["addItem"],
//        groups: [{
//            hidden: false,
//            root: "Machine.operatingsystem"
//        
//        }]
//    }, {
//        id: "software.details",
//        title: "Software",
//        nColumns: 2,
//      actions: ["addItem"],
//        groups: [{
//            hidden: false,
//            root: "Machine.SoftwareInstall"
//        }]
//    }] 
// }
//
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.TemplatedAssetContentProvider");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal._catalog.Attribute");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetTemplateUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.GroupUtil");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.util.AutoWordWrapping");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var AssetTemplateUtil = com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetTemplateUtil;
var EXCLUDE_ALL_TAG = "RTLM-ALL-ATTRIBUTES"; 
var TYPE_DATE = 'TimeStampType';
var VALIDATION_DISK_RANGE = "Invalid_Disk_Range";
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.TemplatedAssetContentProvider", 
    [com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm.ContentProvider,
     com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener],
{   
    template: null,
    layoutForm: null,
    showPartTitle: false,
    showArrayAttributeTitle: false,
    chainedListener: null,
    checkFieldIsRequired: false,
    autoShowAttributeWithValues: false,
    autoShowUnknownAttributes: false,
    createPartSelectorOnlyWithSubTypes: true,
    
    // Summary:
    //
    constructor: function(layoutForm, formListener, template)
    {
        var obj = template;
        if(dojo.isString(template)) {
            obj = dojo.fromJson(template);
            if(obj == null) {
                throw new Error("Template JSON string cannot be converted to object.");
            }
        }
        this.template = obj;
        
        // setup the attribute treatment directives from the template.
        // 
        // turn on/off the "isRequired" fields
        if(typeof this.template.needIsRequiredCheck != "undefined")
            this.checkFieldIsRequired = this.template.needIsRequiredCheck;
            
        // turn on/off showing unknown attributes
        if(typeof this.template.autoAddUnknownAttributes != "undefined")
            this.autoShowUnknownAttributes = this.template.autoAddUnknownAttributes;
        
        // turn on/off showing attributes with values even if they're hidden
        if(typeof this.template.autoAddAttributeWithValue != "undefined")    
            this.autoShowAttributeWithValues = this.template.autoAddAttributeWithValue;
    
        this.layoutForm = layoutForm;
        this.layoutForm.setListener(this);
        this.chainedListener = formListener;
    },
    
    getTemplateJsonString: function()
    {
       return dojo.toJson(this.template);    
    },
    
    // Summary:
    //    Each section in the template can specify a different "root" attribute.
    //    This allows showing of attributes from different PARTS of the asset in the
    //    same section. This probably not a common use case, but the grouping allows this.
    //    This methods returns true if the specified groupRootName is a root of
    //    of the specified section.
    //
    //      Example:
    //      if in a section there's group defined as such:
    //        groups: [
    //            {
    //                hidden: false,
    //                root: "Machine.operatingsystem"
    //
    //            }
    //        ]
    //    If the caller specified 'Machine.operatingsystem", this method returns true.
    //
    getSectionGroupByRoot: function(sectionKey, groupRootName)
    {
        var groups = sectionKey.groups;
        
        for(var i = 0; i < groups.length; i++) {
            if(this._matchGroupRootName(groups[i], groupRootName)) {
                return groups[i];
            }
        }
        return null;
    },
    
    _matchGroupRootName: function(group, groupRootName)
    {
        if(group.root == groupRootName)
            return true;
        
        if(group.showOnlyWithValue) {
            var qualName = group.root + "." + group.showOnlyWithValue;
            return (qualName.toLowerCase() == groupRootName.toLowerCase())
        }
        return false;
    },
    
    // Summary:
    //    Each root attribute in a group can specify the list of sub-attributes
    //    to be shown for that attribute. If one is defined for the specified root
    //    this method returns the array of attributes to be shown. If no attribute list
    //    is defined, this returns a null array. Optionally, if the sub-attributes list
    //    not define, specifying the 'getDefaultList' parameter to true will generate
    //    the full attribute list. 
    //
    getAttributeDisplayListForSection: function(sectionKey, groupRootName, getDefaultList)
    {
        var groups = sectionKey.groups;
        
        for(var i = 0; i < groups.length; i++) {
            if(this._matchGroupRootName(groups[i], groupRootName)) {
                // return the defined list, if one saved.
                if(groups[i].attributes) {
                    return groups[i].attributes;                    
                }
                // there's no show attribute saved in the template. If we are
                // asked to get the default list, do that. 
                else if(getDefaultList) {
                    var asset = this.layoutForm.getContent();
                    var rootParent = this._getAttributeParent(asset, groupRootName);
                    var attrName = groupRootName.slice(groupRootName.lastIndexOf(".") + 1);
                    var attr = rootParent.getAttribute(attrName);
                    var attrNames = [];
                    
                    // the part selector needs to be in the list too.
                    attrNames.push(attrName);
                    
                    if (attr.getType() == DefaultAssetContentProvider.TYPE_PART) {
                        var part = attr.getRootPart();
                        var subAttrNames = part.getAttributeNames();
                        
                        for(var i = 0; i < subAttrNames.length; i++) {
                            attrNames.push(subAttrNames[i]);
                        }
                        return attrNames;
                    }
                }
            }
        }
        return null;
    },

    _isNameAssetRoot: function(attrName)
    {
        return attrName.indexOf(".") < 0;
    },
    
    _dupArray: function(array)
    {
        var dup = [];
        if(array) {
            for(var i = 0; i < array.length; i++) {
                dup.push(array[i]);
            }
        }
        return dup;    
    },

    _inList: function(list, value)
    {
        if(list && dojo.isArray(list)) {
            for(var i = 0; i < list.length; i++) {
                if(list[i] == value)
                    return i;
            }
        }
        return -1;
    },
    
    // Summary:
    //    Based on the template's settings, add more attributes to the static
    //    attribute list for an Part. There are two settings that augments
    //    the attribute display list of a part. One is based on whether the
    //    attribute has value, and the other is based on whether the attribute
    //    is known at all.
    //
    //    For the case for showing attributes that has value, even if the attribute 
    //    is explicitly being hidden, the attribute would still be showing.
    //    For the case for showing unknown attributes, the attribute is added when
    //    it doesn't appear in both of the exclude list.
    // 
    _getAugmentedAttributeListForPart: function(part, showList, hiddenList, excludeList)
    {
        var isExcludeAll = this._inList(excludeList, EXCLUDE_ALL_TAG) > -1;

        // we're not doing addative to our static attribute list,
        // or we're excluding all other attributes, just return 
        // the static list in the template. 
        if(!this.autoShowAttributeWithValues && !this.autoShowUnknownAttributes) {
            return showList;
        }
        var resultList = this._dupArray(showList);
        var allAttrs = part.getAttributeNames();
        
        for(var i = 0; i < allAttrs.length; i++) {
            // this attribute is already in our list,
            // don't need to consider it.
            if(this._inList(showList, allAttrs[i]) > -1)
                continue;
            
            if(this.autoShowAttributeWithValues) {
                // Only bring in the attributes w/ values that we know, that's if it
                // is in the hidden list only.
                if(this._inList(hiddenList, allAttrs[i]) >= 0) {
                    var attr = part.getAttribute(allAttrs[i]);
                    
                    if(attr.isArray() && attr.getArrayValues().length > 0) {
                        resultList.push(allAttrs[i]);
                        continue;
                    }
                    if(attr.getValue() != null) {
                        var val = attr.getValue();
                        
                        if(attr.getType() == DefaultAssetContentProvider.TYPE_PART) {
                            if(val.getName() != attr.getRootPart().getName())
                                resultList.push(allAttrs[i]);
                                continue;
                        }
                        // if the value is an object, it has value, 
                        // or if it is a string, ensure that it's
                        // not empty.
                        if(dojo.isObject(val) || (dojo.isString(val) && val.length > 0)) {
                            resultList.push(allAttrs[i]);                       
                            continue;
                        }
                    }
                }
            }
            if(this.autoShowUnknownAttributes) {
                // only add this attribute if we really don't know it, that
                // is it's not in our hidden and exclude list.
                if(this._inList(hiddenList, allAttrs[i]) == -1 &&
                   !isExcludeAll &&
                   this._inList(excludeList, allAttrs[i]) == -1) {
                    if(hiddenList != null) {
                        hiddenList.push(allAttrs[i]);
                    }
                    else {
                        resultList.push(allAttrs[i]);        
                    }  
                }
            }
        }
        return resultList;
    },
    
    // Summary:
    //    Each root attribute in a group can specify the list of sub-attributes
    //    to be shown for that attribute. Certain sub-attributes from that list
    //    can be hidden. This method returns the hidden sub-attributes, or null
    //    if null is defined.
    //
    getAttributeHiddenListForSection: function(sectionKey, groupRootName)
    {
        var groups = sectionKey.groups;
        
        for(var i = 0; i < groups.length; i++) {
            if(groups[i].root == groupRootName) {
                if(groups[i].hiddenAttributes) {
                    return groups[i].hiddenAttributes;                    
                }
                break;
            }
        }
        return null;
    },
    
    // Summary:
    //    Updates the section's show/hidden attributes for the specified attribute root.
    //    This method also invalidates the section in question in the form to reflect the
    //    new attribute settings. In addition, this method also triggers the saving of the 
    //    template for the user.
    //
    setShowHideAttributeForSection: function(sectionKey, groupRootName, showList, hideList)
    {
        var groups = sectionKey.groups;
        
        for(var i = 0; i < groups.length; i++) {
            if(groups[i].root == groupRootName) {
                groups[i]["attributes"] = showList;
                groups[i]["hiddenAttributes"] = hideList;
                
                this.layoutForm.invalidateSectionLayout(sectionKey);
                AssetTemplateUtil.saveTemplate(this.template);
                break;
            }
        }        
    },
    
    // Summary:
    //    The template provider will "merge" in the attributes that has value, but
    //    currently is not on the attribute display list. 
    //
    setAutoShowAttributeWithValues: function(show)
    {
        this.autoShowAttributeWithValues = show;
    },

    // Summary:
    //    The template provider will "merge" in the attributes that are not known
    //    in the current template. Unknown attributes mean that they don't appear
    //    in either the attributes and hiddenAttributes list of a section group but
    //    are in the schema model.
    //
    setAutoShowUnknownAttribute: function(show)
    {
        this.autoShowUnknownAttributes = show;
    },
    
    setCreatePartSelectorOnlyWithSubTypes: function(createOnlyWithSubTypes)
    {
        this.createPartSelectorOnlyWithSubTypes = createOnlyWithSubTypes;
    },
    
    // Summary:
    //    Indicate to the provider to show the attribute's title if it's a PART type.
    //    PART attributes can contain other attributes. If this is set to true, a title
    //    is created for the collection of attributes that is under a PART attribute. By
    //    default, PART type attribute title are not shown.
    //
    setShowPartTypeTitle: function(show)
    {
        this.showPartTitle = show; 
    },
    
    // Summary:
    //    Indicate to the provider to show the attribute's title if it's an array type.
    //    Array attributes holds mulitple values for a single attribute. If this is set to true, a title
    //    is created for the collection of values for array attributes. By default, array attribute
    //    title are set to show.
    //
    //    NOTE: Please note that array attributes are most likely a PART attribute as well. Subclass of this 
    //    provider should only enable one title display. 
    //
    setShowArrayAttributeTitle: function(show)
    {
        this.showArrayAttributeTitle = show; 
    },

    // Summary:
    //    Returns the layout information about the form. The following is the info format:
    //    {
    //         nColumns: integer > 0,  the number of sections across the form layout. 
    //         tileDirection: value,   SectionLayoutForm.TILE_VERTICAL, or SectionLayoutForm.TILE_HORIZONTAL
    //         sections: ["key",...]   the array of section keys** (please see below)
    //    }
    //    If the tiling direction is vertical, the sections in the info object are filled from
    //    column 1 first. Otherwise, for horizontal tiling, the sections are filled from
    //    row 1 first.
    //
    //    ** The section keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung section ids. The section's key (id) can then be used to reference and access the
    //    section layout.
    //
    getFormInfo: function(/* the content object */content) 
    {
        if(!this.template) {
            
        }
        var info = {};
        info["nColumns"] = 1;
        info["tileDirection"] = SectionLayoutForm.TILE_VERTICAL;
        info["sections"] = this.template["sections"];
        return info;
    },
    
    // Summary:
    //    Returns information about a section given its key value. The following is the info format:
    //    {
    //         title: "string",          the title for the section
    //         nColumns: integer > 0,    the number of columns of attributes across the section. 
    //         attributes: ["key",...]   the array of attribute keys** (please see below)
    //    }
    //
    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique among attribute ids. The attribute's key (id) can then be used to reference and access the
    //    attribute layout.
    //
    getSectionInfo: function(/* the content object */content, sectionKey)
    {
        var aSection = {};
        
        RtlmUiUtil.assertExists(sectionKey.title, "section.title");
        RtlmUiUtil.assertExists(sectionKey.nColumns, "section.nColumns");
        RtlmUiUtil.assertExists(sectionKey.groups, "section.groups");
        
        aSection["title"] = sectionKey.title;
        aSection.nColumns = sectionKey.nColumns;
        var attrKeys = [];
        
        var groups = sectionKey.groups;
        
        for(var i = 0; i < groups.length; i++) {
            this._processSectionAttrGroup(content, groups[i], attrKeys);
        }
        aSection["attributes"] = attrKeys;
        return aSection;        
    },
    
    _processSectionAttrGroup: function(asset, group, attrKeys)
    {
        RtlmUiUtil.assertExists(group.root, "section.groups[].root");
        RtlmUiUtil.assertExists(group.hidden, "section.groups[].hidden");

        var root = group.root;
        var isHidden = group.hidden;
        var part = this._getAttributeByPath(asset, root);
        
        if(part) {
            if(isHidden && group.attributes) {
                var keys = this._createAttributeList(root, part, group.attributes, group.hiddenAttributes, group.excludeAttributes);
                
                for(var i = 0; i < keys.length; i++)
                    attrKeys.push(keys[i]);
            }
            else {
                var rootParent = this._getAttributeParent(asset, root);
                var attrName = root.slice(root.lastIndexOf(".") + 1);
                var rootQualifier = (typeof group.rootQualifier == "undefined") ? null : group.rootQualifier;
                var aKey = this._createAttributeKey(rootParent, attrName, rootQualifier);
                attrKeys.push(aKey);
            }
        }        
    },
    
    _getAttributeParent: function(asset, attributePath)
    {
        var parentPos = attributePath.lastIndexOf(".");
        var parentPath = attributePath.substr(0, parentPos);
        return this._getAttributeByPath(asset, parentPath);
    },
    
    _getAttributeByPath: function(asset, attrPath)
    {
		var part = null;
        if (asset instanceof com.ibm.asq.common.web.client.internal._catalog.Part) 
		{
			part = asset;
		}
		else 
		{
			var rootAtt = asset.getRootAttribute();
			if (rootAtt.isArray()) 
			{
				// Note, this assumption of value[0] will not hold in the future.
				part = rootAtt.getArrayValue(0);
			}
			else 
			{
				part = rootAtt.getValue();
			}
		}
        
        var paths = attrPath.split(".");
        var i = 0;
        var attr;
        
        while(i < paths.length - 1) {
            try {
                attr = part.getAttribute(paths[i + 1]);
                
                // we're not at the attribute we wanted yet
                // Ensure this is a valid part.
                if((i + 1) < paths.length) {
                    if(attr.getType() != DefaultAssetContentProvider.TYPE_PART) {
                        throw new Error("Invalid attribute Path: " + attrPath);
                    }
                    part = attr.getRootPart();
                }
                i++;
            }
            catch(error) {
                throw new Error("Invalid attribute Path: " + attrPath);
            }
        }
        return part;
    },
    
    
    // Summary:
    //    Returns information about the attribute given its key value. The attribute info is not static for
    //    all attributes being described. Certain attribute types will have additional data 
    //    associated with them.
    //
    //    The following is the info format:
    //    {
    //         /* 
    //          * Required Info Data 
    //          */
    //         type: value,              the type of attribute, one of the SectionLayoutForm.ATTRIBUTE_*
    //
    //         /* 
    //          * Optional Info Data (NOT YET IMPLEMENTED)
    //          */
    //         colSpan: <1..section.nCol> Optionally overrides the parent section layout's nCols attribute.
    //                                    By default, the attribute will occupy one column of the section. This
    //                                    allows the attribute to occupy more then one column in the section.
    //                                    For attribute type ATTRIBUTE_COLLECTION, the default is section.nCols. 
    //                                    It occupies the entire row in the section.
    //
    //         /* 
    //          * Common Info Data for "atomic" attribute types
    //          */
    //         title: "string",          the title for the attribute
    //         readOnly: <true,false>,  
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_COLLECTION type
    //          */
    //         collection: [keys...],    the array of sub-attribute keys** (please see below)
    //         title: "string",          Optional, the title for the collection
    //
    //         /* 
    //          * Specific Info Data for the SectionLayoutForm.ATTRIBUTE_ENUM type
    //          */
    //         enumChoices:[values...]
    //    }
    //
    //    ** The attribute keys can be a simple string tag, or an opique object. If it's an opique
    //    object, it must have an id attribute, like key.id that is a string value. The key (or id) must be
    //    unique amoung ALL attribute ids. The attribute's id can then be used to reference and access the
    //    attribute layout.
    //
    getAttributeInfo: function(/* the content object */content, sectionKey, attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = null;
        var attrInfo = {};
        
        if(parentPart) {
            attr = parentPart.getAttribute(attrName);
        }
        
        // handle the special pseudo attribute type for the part selection
        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
            return this._constructPartTypeSelectorAttrInfo(attributeKey, parentPart, attr);
        }

        // handle multi-value attribute intances. Create the 
        // concrete attribute info for an array value in the attribute.
        if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE) {
            attrInfo = this._constructAttributeInfo(sectionKey, attributeKey, parentPart, attr);
            var canDelete = true;
            
            if(sectionKey.groups[0].rootIsReadOnly) {
                if(attr.getPathName() == sectionKey.groups[0].root) {
                    canDelete = false;
                }
            }
            attrInfo["canDelete"] = canDelete;
            
            // provide a specific delete message about the attribute.
            if(canDelete) {
                var name = RtlmUiUtil.getMessage("pl_" + attr.getName());
                
                if(attr.getType() == DefaultAssetContentProvider.TYPE_PART) {
                    var part = attr.getArrayValue(attributeKey.index);
                    if (part) 
                        name = part.getName();
                }
                attrInfo["deleteTooltip"] = RtlmUiUtil.getMessage("labelDeleteButton") + " " + name;
            }    
            attrInfo["separator"] = true;
            return attrInfo;
        }
       
        var isMultiValueCollection = attr.isArray();
        
        // Create a collection to hold the array values for this attribute.
        if(isMultiValueCollection) {
            return this._constructMultiValueAttrInfo(sectionKey, attributeKey, parentPart, attr);
        }
        
        return this._constructAttributeInfo(sectionKey, attributeKey, parentPart, attr);
       
    },
    
    _constructPartTypeSelectorAttrInfo: function(attributeKey, parentPart, attr)
    {
        var attrInfo = {};
        
        attrInfo["type"] = SectionLayoutForm.ATTRIBUTE_ENUM;
        attrInfo["readOnly"] = false;
        
        if(attributeKey["isRootSelector"]) {
            var asset = this.layoutForm.getContent();
            var choices = [];
			var rootName;
			var subTypes;
			if (asset instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
                // part!, not asset
                rootName = asset.getName();
                subTypes = asset.getTemplate().getPartNames();
			}else{
                var rootAttr = asset.getRootAttribute();
                rootName = rootAttr.getRootPart().getName();
                subTypes = rootAttr.getAllPartTypeNames();              
			}
            
            choices.push(rootName);
            for(var i = 0; i < subTypes.length; i++)
                choices.push(subTypes[i]);
                
            attrInfo["enumChoices"] = choices;
            attrInfo["title"] = RtlmUiUtil.getMessage("root");
        }
        else {
            var title = RtlmUiUtil.getMessage(attr.getName() + "Selector");
            attrInfo["enumChoices"] = attr.getAllPartTypeNames();
            attrInfo["defaultSelection"] = attributeKey.selectedPartName;
            attrInfo["title"] = title;
            
            // always create an empty choice
            attrInfo["createFirstEmpty"] = true;
        }        
        return attrInfo;
    },
    
    _constructMultiValueAttrInfo: function(sectionKey, attributeKey, parentPart, attr)
    {
        var attrInfo = {};
        var attrName = attributeKey.attributeName;
        var title = (this.showArrayAttributeTitle) ? RtlmUiUtil.getMessage(attrName) : null;
        
        attrInfo["type"] = SectionLayoutForm.ATTRIBUTE_COLLECTION;
        attrInfo["collection"] = this._createMultiValueList(sectionKey, attributeKey, parentPart, attr);
        attrInfo["title"] = title;
        attrInfo["readOnly"] = false;
        return attrInfo;
    },
    
    _constructAttributeInfo: function(sectionKey, attributeKey, parentPart, attr) 
    {
        var attrName = attr.getName();
        var type = attr.getType();
        var subAttributeNames = [];
        var layoutType = SectionLayoutForm.ATTRIBUTE_LABEL;
        var attrInfo = {};
        var showTitle = true;
        var title = RtlmUiUtil.getMessage(attrName);
        
        if(type == DefaultAssetContentProvider.TYPE_PART) {
            var part;
            var subAttrs = [];
            var index = attributeKey["index"];
            var needPartSelector = true;
            
            if(attr.isArray()) {
                var values = attr.getArrayValues();
                part = values[index];
            }
            else {
                part = attr.getValue();
            }
            
            // an instance part is set. Grab its value.
            if(!part) {
                // there's no value set in the attribute, ask its parent
                // to create a temporary root part for now. We'll show
                // the root part's attributes. 
                part = parentPart.getAttributeValue(attrName, true);
            }
            
            if(this.createPartSelectorOnlyWithSubTypes) {
                var subParts = attr.getAllPartTypeNames();
                needPartSelector = (subParts && subParts.length > 0);
            }
            
            // create a pseudo sub-attribute for this PART attribute. It's
            // an ENUM attribute that shows the part's instance types. This
            // allows the user to select the instance types of the part
            //
            var uniqueId = attributeKey.id + attrName + "Selector";

            var partSelAttr = {};
            partSelAttr["id"] = uniqueId;
            partSelAttr["type"] = DefaultAssetContentProvider.TYPE_PART_SELECTOR;
            partSelAttr["parentKey"] = attributeKey;
            partSelAttr["parentPart"] = parentPart;
            partSelAttr["attributeName"] = attrName;

            if(attr.isArray()) {
                partSelAttr["index"] = index;
            }
                           
            var subAttrs = null;
            
            // Use the specific part instance, or the root part to get its attributes.
            // By default, we'll always show the root part's attribute even if no instance
            // was specified.
            if(part) {                
                // if this attribute is an root in the section's group,
                // get its filter list out, if there's one.
                var group = this.getSectionGroupByRoot(sectionKey, attr.getPathName());
                if(group) {
                    var subAttrList = null;
                    var excludeList = group.excludeAttributes;

                    // The "effective" attribute is the full qualifier name.
                    // The exclude list applies only to the effective's parent and
                    // not to the attribute identified by "showOnlyWithValue".
                    //
                    if(group.showOnlyWithValue) {
                        var qualName = group.root + "." + group.showOnlyWithValue;
                        if(attr.getPathName().toLowerCase() == qualName.toLowerCase())
                            excludeList = null;
                    }
                    subAttrList = this.getAttributeDisplayListForSection(sectionKey, attr.getPathName());
                    subAttrs = this._createPartAttributeListForCollection(part, attributeKey, subAttrList, group.hiddenAttributes, excludeList, index);
                    
                    if(!subAttrList) {
                        if(needPartSelector)
                            subAttrs.unshift(partSelAttr);
                    }
                }
                else {
                    subAttrs = this._createAttributeList(attr.getPathName(), part, null, null, null, attributeKey);
                    if(needPartSelector)
                        subAttrs.unshift(partSelAttr);
                }
            }

            layoutType = SectionLayoutForm.ATTRIBUTE_COLLECTION;
            attrInfo["collection"] = subAttrs;            
            
            // turn on/off title based on the provider setting. We can
            // override this if the attribute is read only and the attribute
            // to show in that attribute has values.
            showTitle = this.showPartTitle; 
            if(sectionKey.groups[0].rootIsReadOnly) {
                if(attr.getPathName() == sectionKey.groups[0].root) {
                    if (sectionKey.groups[0].showOnlyWithValue) {
                        var attr = part.getAttribute(sectionKey.groups[0].showOnlyWithValue);
                        if (attr.getArrayValues().length > 0) {
                            title = RtlmUiUtil.getMessage(part.getName());
                            showTitle = true;
                        }
                    }                        
                }
            }
        }
        else if(attr.isEnum()) {
            layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
            attrInfo["enumChoices"] = attr.getEnumValues();
            
            // always create an empty choice
            attrInfo["createFirstEmpty"] = true;
        }
        else if(type == DefaultAssetContentProvider.TYPE_STRING) {
            // special case to create an IP field 
            // for the ip address attribute.
            if(attrName == "ipAddress") {
                layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_IP;
            }
            // special case for the machine name
            else if(attrName == "name" && parentPart.getType() == "Machine" && attr.isAsset()) {
                layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_CHECKBOX;
                var ttMessage = RtlmUiUtil.getMessage("machDtlPopHostname");
                attrInfo.tooltipMessage = ttMessage;
            }
            // special case for the hostname
            else if(attrName == "hostname" && parentPart.getType() == "OperatingSystem" && attr.isAsset()) {
                layoutType = SectionLayoutForm.ATTRIBUTE_TEXT;
                var assetNameAttribute = this.template.assetType+".name";
                var nameAttrInfo = this.layoutForm.attributeMap[assetNameAttribute];
                
                if(nameAttrInfo) {
                    var cb = nameAttrInfo.info.checkbox;
                    if (cb && cb.checked && (attributeKey.index == null || attributeKey.index == 0)) {
                        if (nameAttrInfo) {
                            var value = nameAttrInfo.node.currentValue;
                            
                            // seed the value into the asset PART as well.
                            var hname = parentPart.getAttributeValue(attrName, false);
                            if(!hname || hname.length == 0) {
                                attrInfo.initValue = value;
                                parentPart.setAttributeValue(attrName, value);
                            }
                        }
                    }
    
                    if (cb && attributeKey.index > 0) {
                        cb.setDisabled(true);
                    }
                }
            }
            else {
                layoutType = SectionLayoutForm.ATTRIBUTE_TEXT;              
            }
        }
        else if(type == DefaultAssetContentProvider.TYPE_NUMBER) {
            layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_NUMERIC;
        }
        else if(type == TYPE_DATE) {
            layoutType = SectionLayoutForm.ATTRIBUTE_TEXT_DATE;
        }
        else if(type == DefaultAssetContentProvider.TYPE_POINTER) {
            // Special case for the owner and group attributes. We'll
            // have to generate the list for its dropdown.
            if(attrName == "owner") {
                layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
                var userNames = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUserNames();
                attrInfo["enumChoices"] = userNames;
            
                // always create an empty choice
                attrInfo["createFirstEmpty"] = true;
            }
            else if(attrName == "group") {
				layoutType = SectionLayoutForm.ATTRIBUTE_ENUM;
				var groupNames = GroupUtil.getAllGroupNames();
				attrInfo["enumChoices"] = groupNames;
					
				// always create an empty choice
				attrInfo["createFirstEmpty"] = true;
				
				//The user may not have permission to alter the group to which a machine belongs to.
				//this is a different permission than the permission to edit the machine.
				//If they do not have permission then disable the select control
    			var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
				var resGroupPerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_RESOURCE_GROUP);
				if (resGroupPerm && !resGroupPerm[PermissionsClient.ACTION_CREATE])
					attrInfo["disabled"] = true; 
            }
        }
        else {
            // print a debug information about unknown field type coming from the
            // server. We have seen empty fields (Label) when it's suppose to be string
            // types. If this hit again, at least we know what's the type being asked for.
            console.debug("Unknown attribute field type: " + type);
        }
		var wrappedTitle = com.ibm.asq.common.web.ui.internal.util.wordWrapping.wrapText(
                        title, 
                        "", 
                        85, // hard code the max width since the dom node has not been added to the tree. 
                        true,
                        "<br/>");
        attrInfo["title"] = (showTitle) ? wrappedTitle : null;
        attrInfo["type"] = layoutType;
        attrInfo["readOnly"] = false;
        
        // apply the isRequire decoration if this template requires it.
        if(this.checkFieldIsRequired) {
            attrInfo["isRequired"] = attr.isRequired();        
        }
        return attrInfo;
    },
    
    // Summary:
    //    Returns the value of the attribute given the attrubute key. 
    //
    getAttributeValue: function(/* the content object */content, attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = (parentPart) ? parentPart.getAttribute(attrName) : null;
        var value = null; 
        
        try {
            if(attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
                if(attributeKey["isRootSelector"]) {
                    var content = this.layoutForm.getContent();
                    // The content differs based on whether we are being called
                    // by the machine form or the config form.  This probably needs
                    // to be fixed, but for now we will just check and get the document.
                    var config = null;
                    if(content instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
                        config = content.document;
                    }
                    else {
                        config = content;
                    }
                    var rootAtt = config.getRootAttribute();
                    if(rootAtt.isArray()) {
                        value = rootAtt.getArrayValueByTag(attributeKey.tag).getName();
                    }
                    else {
                        value = rootAtt.getValue().getName();
                    }
                }                
                else {
                    var partInst = this._internalGetAttributeValue(attributeKey);
                    
                    if(partInst) {
                        // We've "auto" instantiated the parent part to default to
                        // show the root's attributes. Check to see if the part value
                        // is set to the root, if so, don't show it.
                        // 
                        var rootPart = attr.getRootPart();
                        
                        if(rootPart.getName() != partInst.getName())
                            value = partInst.getName();             
                    }
                }
            }
            else {                
                var type = attr.getType();
                value = this._internalGetAttributeValue(attributeKey);
                
                // a pointer object has two components. Use
                // the display string for value.
                if(type == DefaultAssetContentProvider.TYPE_POINTER && value) {
                    var pointer = value;
                    value = pointer.getDisplayString();
                }
            }
        }
        catch(error) {
            console.debug("Error occurred: " + error);
            value = null;
        }
        return value;
    },
    
    // Summary:
    //    Gets the value of the specified attribute. If the attribute is
    //    an array attribute, returns the array value, or if it
    //    is a single value attribute, returns the value. In either case
    //    if the value doesn't have value set, it returns null.
    //
    _internalGetAttributeValue: function(attributeKey)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var value = null;
        
        if(attr.isArray()) {
            var index = attributeKey["index"];
            var values = attr.getArrayValues();
            
            // sanity check. We built the key with the attribute array value;
            // it *should* have the value in there.
            if(values && index < values.length)
                value = values[index];
        }
        else {
            value = attr.getValue();
        }
        return value;
    },

    // Summary:
    //    Set the value of the specified attribute. Call the proper attribute
    //    method to set the value based on whether the attribute is an array
    //    attribute.
    //
    _internalSetAttributeValue: function(attributeKey, value)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
            
        // If this is a "clear" value, replace the value
        // with its root part. Otherwise, the attribute will
        // prune its value if it's null.
        //
        if(attr.getType() == DefaultAssetContentProvider.TYPE_PART && (value == null || value.length < 1)) {
            var rootPart = attr.getRootPart();
            value = rootPart.getName();             
        }
            
        if(attr.isArray()) {
            var index = attributeKey["index"];
            
            attr.setArrayValue(index, value);
        }
        else {
            attr.setValue(value);
        }
    },

    _mapAttributeValidationErrorMessage: function(attribute, validateVal)
    {
        switch(validateVal) {
            case attribute.VALIDAIION_INVALID_CHARACTER: return RtlmUiUtil.getMessage("msgAttrInvalidChar");
            case attribute.VALIDATION_INVALID_SEQUENCE: return RtlmUiUtil.getMessage("msgAttrInvalidSequence");
            case attribute.VALIDATION_DUPLICATE_OPERATORS: return RtlmUiUtil.getMessage("msgAttrDupOps");
            case attribute.VALIDATION_DISJOINTED: return RtlmUiUtil.getMessage("msgAttrDisjoined");
            case attribute.VALIDATION_INCOMPLETE: return RtlmUiUtil.getMessage("msgAttrIncomplete");
            case attribute.VALIDATION_EXTRA_COMPONENTS: return RtlmUiUtil.getMessage("msgAttrExtraComp");
            case VALIDATION_DISK_RANGE: return RtlmUiUtil.getMessage("msgAttrInvalidDiskSpaceRange");
            default: return RtlmUiUtil.getMessage("msgInvalidValue");
        }
    },
    
    _moreValidation: function(parentPart, attr, newVal)
    {
        var totalVal = -1;
        var freeVal = -1;
        
        if(attr.getName() == "TotalDiskSize") {
            var attVal = parentPart.getAttribute("FreeDiskSpace").getValue();
            if(attVal)
                freeVal = parseInt(attVal);
            totalVal = newVal;
        }
        if(attr.getName() == "FreeDiskSpace") {
            var attVal = parentPart.getAttribute("TotalDiskSize").getValue();
            freeVal = newVal;
            if(attVal)
                totalVal = parseInt(attVal);
        }
        if(totalVal > -1 && freeVal > -1 && totalVal < freeVal) {
            return VALIDATION_DISK_RANGE;
        }
        return attr.VALIDATION_OK;
    },
    
    attributeValueChanged: function(attributeKey, oldValue, newValue)
    {
        if (oldValue == newValue) {
            // skip any onkey event which trigger this as well that actually has no value changed
            return;
        }
        
        var action = {"edited":true};
        
        if (attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR && attributeKey["isRootSelector"]) {
            var asset = this.layoutForm.getContent();
            if (asset instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
                var parts = asset.document.getRootAttribute().getArrayValues();
				var tag = asset.getTag();
                for (var i = 0; i < parts.length; ++i){
                    if (tag == parts[i].getTag()){
                        // found my part!
                        asset.document.setRootPart(i,newValue);
                    }
                }
            }else{
                var rootAtt = asset.getRootAttribute();
                if (rootAtt.isArray()) {
                    rootAtt.setArrayValueByTag(attributeKey.tag, newValue);
                }
                else {
                    asset.setRoot(newValue);
                }
			}
            this.layoutForm.doLayout();
            this.layoutForm.loadDataToForm();
        }
        else {
            var parentPart = attributeKey.parentPart;
            var attrName = attributeKey.attributeName;
            var attr = parentPart.getAttribute(attrName);
            var value = (newValue && newValue.length) ? newValue : null;
            
            // handle the owner and group attribute differently. We have to
            // pass in the uuid as well.
            if (attrName == "owner" && value) {
                var user = {
                    "ds": newValue,
                    "uuid": this._getUserIdByName(newValue)
                };
                value = user;
            }
            else if (attrName == "group" && value) {
                var group = {
                    "ds": newValue,
                    "uuid": this._getGroupIdByName(newValue)
                };
                value = group;
            }

            var attrValid = attr.validate(value);
            
            if(attrValid == attr.VALIDATION_OK) {
                attrValid = this._moreValidation(parentPart, attr, newValue);
            }
            if (attrValid == attr.VALIDATION_OK) {
                // update the attribute with the new value.
                this._internalSetAttributeValue(attributeKey, value);
                
                // This is a part attribute and it has a new instance, 
                // invalidate its layout.
                if (attributeKey["type"] && attributeKey["type"] == DefaultAssetContentProvider.TYPE_PART_SELECTOR) {
                    this.layoutForm.invalidateAttributeLayout(attributeKey.parentKey);
                    this._updateRelatedSections(attr.getPathName());
                }
            } else {
                // the new value is invalid
                action = {"hasError": true, "errorMessage": this._mapAttributeValidationErrorMessage(attr, attrValid)};
            }
            
            // If the attribute is the machine name, we may need to poulate the hostname
            if (attrName == "name" && parentPart.getType() == "Machine"){
                var cb = this.layoutForm.attributeMap[attributeKey.id].info.checkbox;
                if (cb && cb.checked) {
                
                    // Check if we have an OS, if we have "one", set the hostname.
                    var osAtt = parentPart.getAttribute("operatingsystem");
                    var oss = osAtt.getArrayValues();
                    var os = null;
                    if (oss.length == 1) {
                        os = osAtt.getArrayValue(0);
                        os.setAttributeValue("hostname", value);
                        this.layoutForm.invalidateSectionLayout("os.details");
                    }
                }
            } 
        }
        
        // update the field status in the form - error or edited
        this.layoutForm.setAttributeFieldStatus(attributeKey, action);

        // if we have another listener wanting to know the event, 
        // forward the event to it.
        if(this.chainedListener) {
            this.chainedListener.attributeValueChanged(attributeKey, oldValue, newValue);
        }
        
        // the field status is updated explicitly so no need to return the action to update the field
        return null;
    },
    
    attributeDeleteActionInvoked: function(attributeKey)
    {
		
        if (this.chainedListener && this.chainedListener.hasConfirmDeleteAttribute && this.chainedListener.hasConfirmDeleteAttribute(attributeKey) ) {
            this.chainedListener.attributeDeleteActionInvoked(attributeKey, dojo.hitch( this, this._deleteAttribute ) );			
		}
        else 
		{
			this._deleteAttribute( attributeKey );
	        // if we have another listener wanting to know the event, 
	        // forward the event to it.
			if(this.chainedListener)
            	this.chainedListener.attributeDeleteActionInvoked(attributeKey);
        }
    },
	
	_deleteAttribute: function( attributeKey )
	{
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);

        // sanity check...
        if(attr.isArray()) {
            var index = attributeKey["index"];
            if(typeof index != "undefined") {
                attr.removeArrayValue(index);
                
                this._updateRelatedSections(attr.getPathName());
                this.layoutForm.invalidateAttributeLayout(attributeKey.parentKey);
            }
            else {
                console.debug("An array attribute value has undefined index value!")                
            }
        }   
        else {
            console.debug("cannot delete an non-array attribute value!")
        } 
        
        // Special case code to enable the name-hostname sych button
        if(attrName == "operatingsystem"){
            if(attr.getArrayValues().length <= 1){
                var attrid = this.template.assetType+".name";
				var attr = this.layoutForm.attributeMap[attrid];
				if (attr){
	                var cb = this.layoutForm.attributeMap[attrid].info.checkbox;
	                if(cb){
	                    cb.setDisabled(false);
	                }
                }
            }
        }    		
	},
    
    _updateRelatedSections: function(attrName) 
    {
        var sections = this.template["sections"];
        if(sections) {
            for(var i = 0; i < sections.length; i++) {
                if(sections[i].groups[0].root == attrName) {
                    this.layoutForm.invalidateSectionLayout(sections[i]);
                }
            }
        }
    },

    // Summary:
    //    Given a PART type, build a list of keys for each of the attributes
    //    that the part holds. This method builds a complete list of attributes
    //    based on the attribute settings in this provider, like auto showing
    //    attributes with values, etc.
    //
    //    @param attrPath - the full pathname of the attribute that holds the part.
    //    @param part - the instance part that is set in the attribute.
    //    @param subAttributeList - the list of attributes to be included (shown)
    //    @param hiddenList - the list of attributes to be hidden from view
    //    @param excludeList - the list of attributes must not show
    //    @param parentIndex - if the attribute is an array type, the index of this value. 
    //
    _createAttributeList: function(attrPath, part, subAttributeList, hiddenList, excludeList, parentKey)
    {
        var attrKeys = [];
        var attributeNames = part.getAttributeNames();
        
        if(subAttributeList) {
            attributeNames = this._getAugmentedAttributeListForPart(part, subAttributeList, hiddenList, excludeList);
        }
        
        var toBottom = [];
        for(var i = 0; i < attributeNames.length; i++) {
            var aKey;
            
            if(attrPath == attributeNames[i]) {
                if(this._isNameAssetRoot(attrPath)) {
                    aKey = {};
                    var parentId = (parentKey) ? parentKey.id : "";
                    aKey["id"] = parentId + attrPath + "Selector";
                    aKey["type"] = DefaultAssetContentProvider.TYPE_PART_SELECTOR;
                    aKey["parentKey"] = null;
                    aKey["parentPart"] = null;
                    aKey["isRootSelector"] = true;
                    aKey["attributeName"] = attrPath;
                    aKey["tag"] = part.getTag();
                    attrKeys.push(aKey);
                }
                continue;
            }
            
            try {
                // sanity check to ensure the name is a value attribute of 
                // the part.
                var attr = part.getAttribute(attributeNames[i]);
                
                // filter down the part attributes to the bottom
                // The Part attribute contains more attributes, they
                // need to be grouped together to make the form more
                // balanced.
                if(attr.getType() == DefaultAssetContentProvider.TYPE_PART) {
                    toBottom.push(attributeNames[i]);
                    continue;
                }
                aKey = this._createAttributeKey(part, attributeNames[i], null, parentKey);
                attrKeys.push(aKey);
            }
            catch(error) {
                console.debug("Ignoring unknown attribute: " + attributeNames[i] + " In part: " + part.getName());
            }
        }
        for(var i = 0; i < toBottom.length; i++) {
            var attr = part.getAttribute(toBottom[i]);
            aKey = this._createAttributeKey(part, toBottom[i], null, parentKey);
            attrKeys.push(aKey);
        }
        return attrKeys;
    },

    // Summary:
    //    Given a PART type, build a list of keys for each of the attributes
    //    that the part holds. 
    //
    _createPartAttributeListForCollection: function(part, attributeKey, subAttributeList, hiddenList, excludeList, parentIndex)
    {
        var parentPart = attributeKey.parentPart;
        var attrName = attributeKey.attributeName;
        var attr = parentPart.getAttribute(attrName);
        var attributeNames = part.getAttributeNames();
        var attrKeys = [];
        
        if(subAttributeList) {
            attributeNames = this._getAugmentedAttributeListForPart(part, subAttributeList, hiddenList, excludeList);
        }
            
        var toBottom = [];
        for(var i = 0; i < attributeNames.length; i++) {
            var aKey;
            
            if(attributeNames[i] == attrName) {
                // create a pseudo sub-attribute for this PART attribute. It's
                // an ENUM attribute that shows the part's instance types. This
                // allows the user to select the instance types of the part
                //
                var index = attributeKey["index"];
                var uniqueId = attributeKey.id + attrName + "Selector";
    
                aKey = {};
                aKey["id"] = uniqueId;
                aKey["type"] = DefaultAssetContentProvider.TYPE_PART_SELECTOR;
                aKey["parentKey"] = attributeKey;
                aKey["parentPart"] = parentPart;
                aKey["attributeName"] = attrName;
                
                if(attr.isArray()) {
                    aKey["index"] = index;
                }
                attrKeys.push(aKey);
            }
            else {
                try {
                    // sanity check to ensure the name is a value attribute of 
                    // the part.
                    var subattr = part.getAttribute(attributeNames[i]);
                    
                    // filter down the part attributes to the bottom
                    // The Part attribute contains more attributes, they
                    // need to be grouped together to make the form more
                    // balanced.
					if (!subattr){
						continue;
					}
                    if(subattr.getType() == DefaultAssetContentProvider.TYPE_PART) {
                        toBottom.push(attributeNames[i]);
                        continue;
                    }
                    aKey = this._createAttributeKey(part, attributeNames[i], null, attributeKey);
                    attrKeys.push(aKey);
                }
                catch(error) {
                    console.debug("Ignoring unknown attribute: " + attributeNames[i] + " In part: " + part.getName());
                }
            }
        }
        // put the part attribute to the bottom.
        for(var i = 0; i < toBottom.length; i++) {
            var attr = part.getAttribute(toBottom[i]);
            aKey = this._createAttributeKey(part, toBottom[i], null, attributeKey);
            attrKeys.push(aKey);
        }
        return attrKeys;
    },
    
    _createAttributeKey: function(part, name, qualifier, parentKey)
    {
        var extraId = (qualifier) ? ("." + qualifier + ".") : "";
        var fullName = part.getAttribute(name).getPathName() + extraId;
        
        // always tag on the parent key's id to ensure uniqueness.
        // The only time where the parentKey is null is we are at
        // the root level.
        if(parentKey) {
            fullName = parentKey.id + name;
        }        
        var uniqueId = fullName;
        var aKey = {};
        aKey["id"] = uniqueId;
        aKey["parentPart"] = part;
        aKey["attributeName"] = name;
        
        if(parentKey && parentKey.index)
            aKey["index"] = parentKey.index;
        
        return aKey;
    },
    
    // Summary:
    //    Given an array attribute, build a list of keys for each of the 
    //    attribute's value.
    //
    _createMultiValueList: function(sectionkey, parentKey, parentPart, attr)
    {
        var attrKeys = [];
        var attrName = attr.getName();
        var values = attr.getArrayValues();
        
        if(values) {
            for(var i = 0; i < values.length; i++) {
                var uniqueId = parentKey.id + attrName + i;
                var aKey = {};
                
                aKey["type"] = DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE;
                aKey["id"] =  uniqueId;
                aKey["parentPart"] =  parentPart;
                aKey["parentKey"] =  parentKey;
                aKey["attributeName"] = attrName;
                aKey["index"] = i;
                attrKeys.push(aKey);
            }           
        }
        return attrKeys;
    },

    _getUserIdByName: function(name)
    {
        var users = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUsers();
        if(users) {
            for(var i = 0; i < users.length; i++) {
                if(users[i].name == name)
                    return users[i].itemId;
            }
        }
        return null;
    },
	
    _getGroupIdByName: function(name)
    {
        var groups = GroupUtil.getAllGroups();
        if(groups) {
            for(var i = 0; i < groups.length; i++) {
                if(groups[i].name == name)
                    return groups[i].itemId;
            }
        }
        return null;
    }
	
});

})();
