/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 // Summary:
 //    The TableRowRenderer is an interface object that the TableViewer uses to
 //    draw a row in the table. The TableViewer has two default implementation of
 //    the row renderer, one is for the default GRID mode and the other for the LIST
 //    mode. Clients of the TableViewer can supply its own row renderer to the TableViewer
 //    as well. The TableViewer has method to allow the client to set a "custom" row 
 //    renderer. However, all row renderer must subclass from, and implements the methods defined
 //    in this interface class.
 //
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableRowRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableRowRenderer",
             null,
{
    // Summary:
    //    Renders a row in the TableViewer's table row. This methods is called
    //    by the TableViewer as it is constructing the table display.
    //     @param viewer the parent viewer that uses this row renderer
    //     @param rowObject the row data to be displayed
    //     @param tr the actual HTML table row dom node that holds the row content.
    //
    renderRow: function(viewer, rowObject, trNode)
    {
        
    },
    
    // Summary:
    //    Certain row renderer implementation might need to cache certain states about
    //    a row. The TableViewer will dynamically contruct and destruct rows as it's performing
    //    actions such as paging and sorting, etc. However, it's desirable for the row's states
    //    to be maintained across these actions. It's up to the row renderer to cache such
    //    states and re-apply them as the same row is being recreated. This method allows the 
    //    TableViewer to instruct the renderer to toss out any cached states, for events such as
    //    new contents is being set, etc.
    //
    clearRowStates: function()
    {
        
    }
});

// Summary:
//   Create a default row renderer to draw our table row. If no custom
//   row renderer is provided, this is the renderer we'll use to draw
//   our table rows.
// 
dojo.declare("TableViewer.DefaultTableRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableRowRenderer,
{
    
    renderRow: function(viewer, rowObject, trNode)
    {
        //console.debug("entering DefaultTableRowRenderer.renderRow");
        
        var provider = viewer.getContentProvider();
        var content = viewer.getContent();
        var groupBy = viewer.getGroupByColumn();
        var nColumns = provider.getNumColumns(content);
        var atCol = trNode.cells.length;
        var slackWidth = 0;
        var nVisible = nColumns;
        
        for (var i = 0; i < nColumns; i++) {
            if(!viewer.isColumnVisible(i) || (groupBy == i)) {
                nVisible--;
            }
        }
        if(nVisible == 0)
            return;
            
        // we're going to remove the group column,
        // distribute its width amongst all columns
        if (groupBy > -1 && viewer.isColumnVisible(groupBy)) {
            slackWidth = Math.floor(viewer.getColumnWidth(groupBy) / nVisible);
        }    
        for(var i = 0; i < nColumns; i++) {
            // skip displaying the group by, or hidden column.
            if(groupBy == i || !viewer.isColumnVisible(i)) {
                continue;
            }
            var type = provider.getCellDisplayType(rowObject, i);
            var data = provider.getCellDisplayData(rowObject, i);
            var extendedInfo = provider.getCellDisplayExtendedInfo(rowObject, i);
            var cellRenderer = viewer.getCellRenderer(type);
            var td = trNode.insertCell(atCol++);
            var colWidth = viewer.getColumnWidthInPixel(i);
            
            if (viewer.isCompactMode) {
                td.nowrap = true;
                td.style.width = colWidth;
                td.style.overflowX = "hidden";
            }
            var node = cellRenderer.renderCell(viewer, data, extendedInfo, viewer.getCellModifier(), rowObject, i, colWidth);
            
            var align = provider.getCellAlignment(rowObject, i);
            if(align == TableConstants.ALIGN_CENTER) {
                td.align = "center";
            }
            else if(align == TableConstants.ALIGN_RIGHT) {
                td.align = "right";   
            }
            td.appendChild(node);
        }
    }
});


})();
